/* HTTP Proxy Server, by Mark Kamichoff [kamicm@rpi.edu] *
 * Original Filename: main.c                             */

#include "includes.h"

int main(int argc, char **argv)
{
	socklen_t clientlength;
	struct sockaddr_in servaddr, clientaddress;
	struct hostent *hptr;
	int on;

	setupsignals();	/* Sets up signal handlers */

	if(argc!=2)
	{
		printf("Syntax: %s <portnum>\n", argv[0]);
		exit(1);
	}

	if(!((atoi(argv[1]) < 65537) && (atoi(argv[1]) > 0 )))
	{
		fprintf(stderr,"Error: Port number %s must be between 0 and 65537\n",
			argv[1]);
		exit(1);
	}

	// Sets up the listening socket for incoming connections
	if(!(listeningfd = socket(AF_INET, SOCK_STREAM, 0)))
	{
		fprintf(stderr,"Error: Socket creation failed, giving up\n");
		exit(1);
	}

	bzero(&servaddr, sizeof(servaddr));		// Zeroes the structure
	servaddr.sin_family = AF_INET;			// IP
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port = htons(atoi(argv[1]));	// Sets the port from cmd-line

	// Calls bind to listen on the specific port
	if(bind(listeningfd, (SA *) &servaddr, sizeof(servaddr)) != 0)
	{
		fprintf(stderr,"Error: bind() returned error for port %s\n",
			argv[1]);
		exit(1);
	}

	/* Set the socket to listen for a connection */

	if(listen(listeningfd, 5) != 0)
	{
		fprintf(stderr,"Error, listen() returned an error\n");
		exit(1);
	}

	while(1)	/* Keeps accepting connections until termination */
	{
		clientlength = sizeof(clientaddress);
		if((clientfd = accept(listeningfd, (SA *) &clientaddress, &clientlength)) < 0)
		{
			fprintf(stderr,"Error: Accept error, dumping nonexistant client...\n");
			continue;
		}

		// Do a reverse-lookup on our client
		hptr = gethostbyaddr((char *) &clientaddress.sin_addr,
			sizeof(clientaddress.sin_addr),
			AF_INET);

		if(fork() == 0)
		{
			close(listeningfd);

			if(doserver(hptr) == -1)
				fprintf(stderr,"Error: doserver() returned -1, client and/or remote server misbehaved\n");

			exit(0);
		}
		close(clientfd);
	}

	exit(0);
}
